@@ -31,6 +31,7 @@ import cairo
 import traceback
 import modrana_utils
 import rectangles
+import gobject
 from configobj import ConfigObj
 from tilenames import *
 
@@ -109,12 +110,12 @@ class MapTiles(ranaModule):
               user configurable
     """
     self.loadRequestCStack = modrana_utils.SynchronizedCircularStack(self.loadRequestCStackSize)
-    self.loadingNotifyQueue = Queue.Queue(1)
+#    self.loadingNotifyQueue = Queue.Queue(1)
     self.downloadRequestPool = []
     self.downloadRequestPoolLock = threading.Lock()
     self.downloadRequestTimeout = 30 # in seconds
     self.startTileDownloadManagementThread()
-    self.startTileLoadingThread()
+    self.idleLoaderActive = False # report the idle tile loader is running
 
     self.shutdownAllThreads = False # notify the threads that shutdown is in progress
     
@@ -161,8 +162,6 @@ class MapTiles(ranaModule):
     
     self.scalingInfo = (scale, z, tileSide)
 
-
-
   def startTileDownloadManagementThread(self):
     """start the consumer thread for download requests"""
     t = Thread(target=self.tileDownloadManager, name='automatic tile download management thread')
@@ -223,28 +222,61 @@ class MapTiles(ranaModule):
         except Exception, e:
           print "exception in tile download manager thread:\n%s" % e
 
-  def startTileLoadingThread(self):
-    """start the loading-request consumer thread"""
-    t = Thread(target=self.tileLoader, name='tile loading thread')
-    t.setDaemon(True) # we need that the worker dies with the program
-    t.start()
+#  def startTileLoadingThread(self):
+#    """start the loading-request consumer thread"""
+#    t = Thread(target=self.tileLoader, name='tile loading thread')
+#    t.setDaemon(True) # we need that the worker dies with the program
+#    t.start()
 
-  def tileLoader(self):
-    """this is a tile loading request consumer thread"""
-    while True:
-      request = self.loadingNotifyQueue.get(block=True) # consume from this queue
-      if request == 'load':
-        # start processing loading requests from the stack
-        while(1):
-          (item,valid) = self.loadRequestCStack.popValid()
-          if valid:
-            (name, x, y, z, layer) = item
-            self.loadImage(name, x, y, z, layer)
-          else:
-            break # the stack is empty
-      elif request == 'shutdown':
-        print "\nmapTiles: tile loading thread shutting down"
-        break
+#  def tileLoader(self):
+#    """this is a tile loading request consumer thread"""
+#    while True:
+#      request = self.loadingNotifyQueue.get(block=True) # consume from this queue
+#      if request == 'load':
+#        # start processing loading requests from the stack
+#        while(1):
+#          (item,valid) = self.loadRequestCStack.popValid()
+#          if valid:
+#            (name, x, y, z, layer) = item
+#            self.loadImage(name, x, y, z, layer)
+#          else:
+#            break # the stack is empty
+#      elif request == 'shutdown':
+#        print "\nmapTiles: tile loading thread shutting down"
+#        break
+
+  def _startIdleTileLoader(self):
+    """add the tile loader as a gobject mainloop idle callback"""
+    if not self.idleLoaderActive: # one idle loader is enought
+      self.idleLoaderActive = True
+      gobject.idle_add(self._idleTileLoaderCB)
+
+  def _idleTileLoaderCB(self):
+    """get loading requests from the circual stack,
+    quit when the stack is empty"""
+    try:
+      # check for modRana shutdown
+      if self.shutdownAllThreads:
+        self.idleLoaderActive = False
+        return False
+      (item,valid) = self.loadRequestCStack.popValid()
+      if valid:
+        (name, x, y, z, layer) = item
+        self.loadImage(name, x, y, z, layer)
+#        print "loaded"
+        return True # dont stop the idle handle
+      else:
+#        print "quiting"
+        self.idleLoaderActive = False
+        return False # the stack is empty, remove this callback
+    except Exception, e:
+      """
+      on an error, we need to shut down or else idleLoaderActive might get stuck
+      and no loader will be started
+      """
+      print("mapTiles: exception in idle loader\n", e)
+      self.idleLoaderActive = False
+      return False
 
   def loadSpecialTiles(self, specialTiles):
     """load special tiles from files to the special tiles cache"""
@@ -550,12 +582,14 @@ class MapTiles(ranaModule):
            list with it - from this list a slice conforming to
            its size limit is taken and set as the new internal list"""
           # notify the loading thread using the notify Queue
-        self.loadingNotifyQueue.put("load", block=False)
+#        self.loadingNotifyQueue.put("load", block=False)
+        # try to start the idle tile loader
+        self._startIdleTileLoader()
 
-    except Queue.Full:
-      """as we use the queue as a notification mechanism, we dont actually need to
-      process all the "load" notifications """
-      pass
+#    except Queue.Full:
+#      """as we use the queue as a notification mechanism, we dont actually need to
+#      process all the "load" notifications """
+#      pass
 
     except Exception, e:
       print "mapTiles: expception while drawing the map layer: %s" % e
@@ -607,15 +641,33 @@ class MapTiles(ranaModule):
 
     # Return the cairo projection to what it was
     cr.restore()
-    
+
+  def _fakePrint(self, text):
+    """print that does nothing"""
+    pass
+
+  def _realPrint(self,text):
+    print(text)
+
   def loadImage(self,name , x, y, z, layer):
     """Check that an image is loaded, and try to load it if not"""
     
     """at this point, there is only a placeholder image in the memmory cache"""
 
+    # check if tile loading debugging is on
+    debug = self.get('tileLoadingDebug', False)
+    if debug:
+      sprint = self._realPrint
+    else:
+      sprint = self._fakePrint
+
+    sprint("###")
+    sprint("loading tile %s" % name)
+
     # first, is it already in the process of being downloaded?
     with self.threadlListCondition:
       if name in self.threads.keys():
+        sprint("tile is being downloaded")
         if(not self.threads[name].finished):
           with self.imagesLock: # substitute the "loading" tile with a "downloading" tile
             downloadingTile = self.downloadingTile
@@ -628,6 +680,7 @@ class MapTiles(ranaModule):
     # seccond, is it in the disk cache?  (including ones recently-downloaded)
     layerInfo = maplayers.get(layer, None)
     if(layerInfo == None): # is the layer info valid
+      sprint("invalid layer")
       return('NOK')
 
     layerPrefix = layerInfo.get('folderPrefix','OSM')
@@ -635,19 +688,26 @@ class MapTiles(ranaModule):
 
     storeTiles = self.m.get('storeTiles', None) # get the tile storage module
     if storeTiles:
+      start1 = time.clock()
       pixbuf = storeTiles.getTile(layerPrefix, z, x, y, layerType)
       """None from getTiles means the tile was not found
          False means loading the tile from file to pixbuf failed"""
       if pixbuf:
+        start2 = time.clock()
         self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
+        if debug:
+          storageType = self.get('tileStorageType', 'files')
+          sprint("tile loaded from local storage (%s) in %1.2f ms" % (storageType,(1000 * (time.clock() - start1))))
+          sprint("tile cached in memmory in %1.2f ms" % (1000 * (time.clock() - start2)))
         return('OK')
 
-    filename = self.getTileFolderPath() + (self.getImagePath(x,y,z,layerPrefix, layerType))
-
     # Image not found anywhere locally - resort to downloading it
+    filename = self.getTileFolderPath() + (self.getImagePath(x,y,z,layerPrefix, layerType))
+    sprint("image not found locally - trying to download")
 
     # Are we allowed to download it ? (network=='full')
     if(self.get('network','full')=='full'):
+      sprint("automatic tile download enabled - starting download")
       # use threads
       """the thread list condition is used to signalize to the download manager,
       that here is a new download request"""
@@ -665,6 +725,8 @@ class MapTiles(ranaModule):
           self.downloadRequestPool.append(request)
 
         self.threadlListCondition.notifyAll() # wake up the download manager
+    else:
+      sprint("automatic tile download disabled - not starting download")
 
   def loadImageFromFile(self,path,name, type="normal", expireTimestamp=None, dictIndex=0):
     pixbuf = gtk.gdk.pixbuf_new_from_file(path)
@@ -711,6 +773,14 @@ class MapTiles(ranaModule):
       if there are too many images, delete them """
       if len(self.images[0]) > self.maxImagesInMemmory:
         self.trimCache()
+      # new tile available, make redraw request TODO: what overhead does this create ?
+      if self.get('tileLoadedRedraw', True):
+        overlay = self.get('overlay', False)
+        if overlay: # only redraw when a composited tile is loaded with overlay on
+          if type == "composite":
+            self.set('needRedraw', True)
+        else: # redraw regardles of type with overlay off
+          self.set('needRedraw', True)
 
   def trimCache(self):
     """to avoid a memmory leak, the maximum size of the image cache is fixed
@@ -791,12 +861,12 @@ class MapTiles(ranaModule):
     # shutdown the worker/consumer threads
     self.shutdownAllThreads = True
 
-    # shutdown the tile loading thread
-    try:
-      self.loadingNotifyQueue.put(('shutdown', ()),block=False)
-    except Queue.Full:
-      """the tile loading thread is demonic, so it will be still killed in the end"""
-      pass
+#    # shutdown the tile loading thread
+#    try:
+#      self.loadingNotifyQueue.put(('shutdown', ()),block=False)
+#    except Queue.Full:
+#      """the tile loading thread is demonic, so it will be still killed in the end"""
+#      pass
     # notify the automatic tile download manager thread about the shutdown
     with self.threadlListCondition:
       self.threadlListCondition.notifyAll()
